@@ -1,4 +1,4 @@
-from __future__ import absolute_import
+
 
 from vertica_python.vertica.messages.message import FrontendMessage
 
@@ -7,4 +7,4 @@ class Terminate(FrontendMessage):
     pass
 
 
-Terminate._message_id('X')
+Terminate._message_id(b'X')
